﻿using System;
using System.Web.UI.HtmlControls;
using SCI.Gateway.Core.BusinessLogic;
using SCI.Gateway.Core.BusinessLogic.Entity;
using SCI.Gateway.Core.BusinessLogic.Protocols;
using SCI.Gateway.Core.BusinessLogic.User;
using SCI.Gateway.InterfaceTypes;
using SCI.Gateway.Web.Controls;
using SCI.Gateway.Web.Utility;
using SCI.Gateway.Web.WorklistPage.Constants;

namespace SCI.Gateway.Web
{
    /// <summary>
    /// This page used to view both the submitted messages letter
    /// and the preview letter.
    /// The page conditionally allows:
    /// Printing of the letter
    /// Saving the letter locally as an HTML file
    /// Storing the letter as an HTML file in Docman (if available)
    /// Changing the status of the message
    /// Importing to a GPASS and other third party applications  
    /// Replying to the letter (eGPFR response)
    /// </summary>
    public class Letter : GatewayPage
    {
        #region GUI Objects

        protected HtmlInputHidden       attachmentList;
        protected SciButton             ChangeStatusButton;
        protected HtmlInputHidden       DocmanLocationCode;
        protected HtmlInputHidden       fldGatewayServices;
        protected HtmlInputHidden       fldSessionKey;
        protected HtmlForm              GatewayForm;
        protected Header                headerControl;
        protected SciButton             ImportButton;
        protected HtmlInputHidden       imported;
        protected HtmlInputHidden       importedSender;
        protected HtmlInputHidden       inbox;
        protected HtmlGenericControl    LetterDiv;
        protected HtmlInputHidden       letterHtml;
        protected HtmlInputHidden       messageID;
        protected HtmlGenericControl    message_xml;
        protected HtmlInputHidden       messageXml;
        protected HtmlInputHidden       patientkey;
        protected HtmlInputHidden       patientName;
        protected SciButton             PrintButton;
        protected SciButton             ReplyButton;
        protected SciButton             SaveToDocmanButton;
        protected SciButton             SaveToFileButton;

        #endregion

        #region Private Variables

        private Boolean                 m_inbox;
        private MessageItem             m_message;
        private MessageType             m_messageType;
        private string                  m_pageTitle;
        private Boolean                 m_preview;
        private ProtocolVersion         m_protocolVersion;
        private DirectoryEntity         m_provider;
        private TempMessage             m_tempMessage;
        private static readonly string ClientScriptUris =
            CreateJavaScriptObject.FromPublicStaticMembers(
                typeof(GatewayLocations),
                new string[]
                        {
                            "BulkPrintProgressUri",
                            "BulkPrintResultsUri",
                            "ChangeStatusUri",
                            "DownloadImageUri",
                            "DownloadUri",
                            "LetterUri",
                            "MessageReplyUri",
                            "ViewMessageUri",
                            "WorklistUri",
                            "GatewayFormName"
                        });

        #endregion

        #region Public Properties

        /// <summary>
        /// Override for page title.
        /// </summary>
        public override string PageTitle
        {
            get { return m_pageTitle; }
        }

        #endregion

        #region Protected Events

        protected override void Page_Load(object sender, EventArgs e)
        {
            base.Page_Load(sender, e);

            if (Request.Params["previewLetter"] != null)
            {
                m_preview = true;
            }

            //setup the client side scripts
            SetVBScriptFiles("scripts",
                             "xml.vbs;" +
                             "DocmanMethodCalls.vbs;" +
                             "ThirdParty.vbs;" +
                             "disletter.vbs;" +
                             "WorklistBulkPrint.vbs",
                             "letterScripts");

            SetJavaScriptFiles("letter.js;" +
                               "changeStatus.js;" +
                               "WorklistControls.js;" +
                               "MessageReply.js");
            ClientScript.RegisterClientScriptBlock(GetType(), "GatewayLocations", ClientScriptUris, true);
            ClientScript.RegisterClientScriptBlock(GetType(), "ChangeStatusFlagKeys", ChangeStatus.ChangeStatusFlagKeysAsJavaScript, true);
            ClientScript.RegisterClientScriptBlock(GetType(), "WorklistFlagKeys", WorklistConstantsAsJavaScript.WorklistFlagKeys, true);
 
            //needed when posting to asp (the change status/flags page)
            fldSessionKey.Value = SessionState.sessionKey;
            fldGatewayServices.Value = SessionState.gatewayServiceURL;

            m_inbox = Convert.ToBoolean(Request.Params["inbox"]);

            if (m_preview)
            {
                RetrieveTransientMessage();
            }
            else
            {
                RetrieveMessage();
            }


            DisplayButtons(); //Conditionally displays relevant buttons

            DisplayLetter(); //populates LetterDiv with letter html

            SetPageTitleText();

            //not concerned with the following submitted letter functonality if previewing
            if (!m_preview)
            {
                //Set hidden field values
                messageID.Value         = m_message.Key.ToString();
                message_xml.InnerHtml   = m_message.MessageXml;
                letterHtml.Value        = m_message.MessageLetterHtml;
                messageXml.Value        = m_message.MessageXml;
                inbox.Value             = m_inbox.ToString();
                imported.Value          = m_message.IsImported.ToString();
                importedSender.Value = m_message.IsImportedSender.ToString();

                //Used within DocmanMethodCalls.vbs
                DirectoryEntity directoryEntity = DirectoryEntity.FindById(SessionState.hceId);
                DocmanLocationCode.Value = directoryEntity.FindDocmanLocationCode();

                WriteClientSideScript();
            }

            SetButtonAvailability();

        }

        /// <summary>
        /// Disables buttons depending on permissions
        /// </summary>
        private void SetButtonAvailability()
        {
            int messageTypeId = m_message.MessageType.Key;
            if(m_preview)
            {
                if(!m_sessionContext.LoggedOnUserHasPermission(UserPermissionDb.SendTab_PrintInProgress, messageTypeId))
                PrintButton.Enabled = false;
            }
            else
            {
                if(m_inbox)
                {
                    if(!m_sessionContext.LoggedOnUserHasPermission(UserPermissionDb.ReceiveTab_PrintReceived, messageTypeId))
                        PrintButton.Enabled = false;

                    if (!m_sessionContext.LoggedOnUserHasPermission(UserPermissionDb.ReceiveTab_SaveToFile, messageTypeId))
                        SaveToFileButton.Enabled = false;

                    if (!m_sessionContext.LoggedOnUserHasPermission(UserPermissionDb.ReceiveTab_FileInDocman, messageTypeId))
                        SaveToDocmanButton.Enabled = false;

                    if (!m_sessionContext.LoggedOnUserHasPermission(UserPermissionDb.ReceiveTab_SetStatusAndFlags, messageTypeId))
                        ChangeStatusButton.Enabled = false;

                    if (!m_sessionContext.LoggedOnUserHasPermission(UserPermissionDb.ReceiveTab_Reply, messageTypeId))
                        ReplyButton.Enabled = false;

                    if (!m_sessionContext.LoggedOnUserHasPermission(UserPermissionDb.ReceiveTab_Import, messageTypeId))
                        ImportButton.Enabled = false;
                }
                else
                {
                    if(!m_sessionContext.LoggedOnUserHasPermission(UserPermissionDb.SendTab_PrintSent, messageTypeId))
                        PrintButton.Enabled = false;

                    if (!m_sessionContext.LoggedOnUserHasPermission(UserPermissionDb.SendTab_SaveToFile, messageTypeId))
                        SaveToFileButton.Enabled = false;

                    if (!m_sessionContext.LoggedOnUserHasPermission(UserPermissionDb.SendTab_FileInDocman, messageTypeId))
                        SaveToDocmanButton.Enabled = false;

                    if (!m_sessionContext.LoggedOnUserHasPermission(UserPermissionDb.SendTab_Import, messageTypeId))
                        ImportButton.Enabled = false;
                }
            }
        }


        protected override void OnInit(EventArgs e)
        {
            InitializeComponent();
            base.OnInit(e);
        }

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            Load += Page_Load;
        }

        #endregion

        #region Private Methods

        private void RetrieveTransientMessage()
        {
            RetrieveBusinessObjects();
            CreateTransientBuiltMessage();
        }

        private void CreateTransientBuiltMessage()
        {
            if (m_tempMessage != null)
            {
                m_message =
                    MessageItem.CreateTransientBuiltMessage(Server.MapPath(GatewayLocations.FolderWebsiteRoot),
                                                            Server.MapPath(GatewayLocations.FolderMessageTypes),
                                                            GatewayUser.FindById(SessionContext.LogonID),
                                                            m_messageType,
                                                            m_provider,
                                                            m_protocolVersion,
                                                            Request.Form);
            }
            else
            {
                MessageItem existingMessage = MessageItem.FindById(Convert.ToInt32(Request.Params["messageID"]));
                m_message = existingMessage.CreateTransientBuiltMessage(
                                        Server.MapPath(GatewayLocations.FolderWebsiteRoot),
                                        Server.MapPath(GatewayLocations.FolderMessageTypes),
                                        Request.Form);
            }
        }

        private void RetrieveBusinessObjects()
        {
            int tempMessageId = Convert.ToInt32(Request["tempMessageId"]);
            if (tempMessageId != 0)
            {
                m_tempMessage       = TempMessage.FindById(tempMessageId);
                m_messageType       = MessageType.FindById(Convert.ToInt32(Request["msgtypeid"]));
                m_provider          = DirectoryEntity.FindById(Convert.ToInt32(Request["providerid"]));
                m_protocolVersion   = ProtocolVersion.FindById(Convert.ToInt32(Request["protocolid"]));
            }
        }

        private void RetrieveMessage()
        {
            m_message = MessageItem.FindById(Convert.ToInt32(Request.Params["messageID"]));
        }

        /// <summary>
        /// Displays the letter html in the LettedDiv on page.
        /// </summary>
        private void DisplayLetter()
        {
            LetterDiv.InnerHtml = m_message.MessageLetterHtml;
        }

        /// <summary>
        /// Displays or hides the letter buttons depending on context
        /// and adds the client side onclick event
        /// </summary>
        private void DisplayButtons()
        {
            //Print button always visible
            PrintButton.Visible = true;
            PrintButton.Attributes.Add("onclick", "PrintLetter(" + m_message.Key + ",'" + m_inbox + "');return false;");

            //the following buttons are not available on the preview letter
            if (!m_preview)
            {
                //Save to File button always visible
                SaveToFileButton.Visible = true;
                SaveToFileButton.Attributes.Add("onclick",
                                                "SaveLetter(" + m_message.Key + ", 'local', " + m_inbox.ToString().ToLower() +", true, false);return false;");

                // Determine if we are filing message and attachments in Docman
                DirectoryEntity directoryEntity = DirectoryEntity.FindById(SessionState.hceId);
                if (directoryEntity.CanFileInDocman(m_message.MessageType))
                {
                    SaveToDocmanButton.Visible = true;
                    // Save to docman click command. 
                    SaveToDocmanButton.Attributes.Add("onclick",
                                                      "SaveLetter(" + m_message.Key + ", 'docman', " + m_inbox.ToString().ToLower() + ", true, false);return false;");
                }
                else
                {
                    SaveToDocmanButton.Visible = false;
                }

                //Change Status button visible if in the inbox
                //Set the viewed flag to true and update the audit log
                if (m_inbox)
                {
                    ChangeStatusButton.Visible = true;
                    ChangeStatusButton.Attributes.Add("onclick", "AlterStatus(" + m_message.Key + ", "
                                                                 + m_message.MessageType.Key + ", "
                                                                 + " 'true' , 'true');return false;");
                    m_message.SetViewed(true);
                }
                else
                {
                    ChangeStatusButton.Visible = false;
                }

                //Import button visible or not as appropriate
                ImportButton.Visible = false;
                if (m_inbox && m_message.MessageType.ImportOnReceived)
                {
                    //Importing received massages is supported for any message type in third party systems and
                    //only for discharges in Gpass.
                    if (SessionState.isThirdParty || (m_message.MessageType.Key == GatewayConstants.DischargeMessageId))
                    {
                        ImportButton.Visible = true;

                        //set up onclick event depending on 3rd party system i.e. GPASS or other
                        if (SessionState.isThirdParty)
                        {

                            ImportButton.Attributes.Add("onclick", "Import3rdParty(\"" + m_message.MessageType.Key + "\", \""
                                                                   + SessionState.thirdPartyPatientKey + "\", "
                                                                   + "document.all.GatewayForm.letterHtml.value, \""
                                                                   + SessionState.thirdPartyProgId + "\", \""
                                                                   + SessionState.thirdpartyApplicationName + "\", \""
                                                                   + m_message.PatientForename.Replace("\"", "\\\"") + "\", \""
                                                                   + m_message.PatientSurname.Replace("\"", "\\\"") + "\", \""
                                                                   + m_message.PatientChi + "\", \""
                                                                   +
                                                                   m_message.PatientDob.GetValueOrDefault().
                                                                       ToShortDateString() + "\", \""
                                                                   + m_message.PatientSex + "\", \""
                                                                   + m_message.MessageType.Name + "\", "
                                                                   + "false, true" +
                                                                   ");return false;");
                        }
                        else
                        {
                            ImportButton.Attributes.Add("onclick", "ImportDischargeToGpass();return false;");
                        }
                    }
                }
                else if (!m_inbox && m_message.MessageType.AutoImportOnSubmit)
                {
                    //Importing submitted messages manually due to failed auto import on submit is supported for
                    //any message type in third party systems
                    if (this.SessionState.isThirdParty)
                    {
                        ImportButton.Visible = true;

                        ImportButton.Attributes.Add("onclick", "Import3rdParty('" + m_message.MessageType.Key + "', '"
                                       + SessionState.thirdPartyPatientKey + "', "
                                       + "document.GatewayForm.letterHtml.value, '"
                                       + SessionState.thirdPartyProgId + "', '"
                                       + SessionState.thirdpartyApplicationName + "', '"
                                       + m_message.PatientForename + "', '"
                                       + m_message.PatientSurname + "', '"
                                       + m_message.PatientChi + "', '"
                                       +
                                       m_message.PatientDob.GetValueOrDefault().
                                           ToShortDateString() + "', '"
                                       + m_message.PatientSex + "', '"
                                       + m_message.MessageType.Name + "', "
                                       + "false, false" +
                                       ");return false;");
                    }
                }

                bool canReply = false;
                if (m_message.MessageType.ReplyMessageType != null)
                {
                    canReply = CurrentUser.UserEntity.CanMakeMessageType(m_message.MessageType.ReplyMessageType);
                }

                //Reply button visible if in the inbox and message has a reply message type  
                //and the reply message is allowed via the Gateway application
                if (m_inbox && (m_message.MessageType.ReplyMessageType != null)
                    && m_message.MessageType.ReplyMessageType.AllowedViaGatewayApplication && canReply)
                {
                    ReplyButton.Visible = true;

                    
                    ReplyButton.Attributes.Add("onclick", "MessageReply(" + m_message.Key + ", "
                                                          + m_message.MessageType.ReplyMessageType.Key + ", "
                                                          + Convert.ToInt32(Request.Params["providerID"]) +
                                                          ");return false;");
                }
                else
                {
                    ReplyButton.Visible = false;
                }
            }
        }

        /// <summary>
        /// Set some client side script on the page that is used when the
        /// "Save to File" or "Save to Docman" options are selected from the worklist
        /// </summary>
        private void WriteClientSideScript()
        {
            string clientScript = "<script language=JScript>\r";

            //need to put try catch around this because of the way the letter is processed for printing
            clientScript += "try{\r";
            clientScript += "SetWorklistViewedIcon();\r";
            clientScript += "}\r";
            clientScript += "catch(e)\r";
            clientScript += "{}\r";
            clientScript += "</script>\r";

            ClientScript.RegisterStartupScript(GetType(), "ClientScript", clientScript);
        }

        /// <summary>
        /// Sets letter page title /// 
        /// </summary>
        private void SetPageTitleText()
        {
            m_pageTitle = m_message.MessageLetterTitle;
        }

        #endregion
    }
}
